<?php

namespace Kadet\Functional;

use Kadet\Functional\Predicate\AllOfPredicate;
use Kadet\Functional\Predicate\AnyOfPredicate;
use Kadet\Functional\Predicate\ClosurePredicate;
use Kadet\Functional\Predicate\ConstantPredicate;

/**
 * @param $predicate
 *
 * @return \Kadet\Functional\Predicate
 */
function predicate($predicate): Predicate
{
    switch (true) {
        case $predicate instanceof Predicate:
            return $predicate;
        case is_callable($predicate):
            return new ClosurePredicate(\Closure::fromCallable($predicate));
        default:
            throw new \InvalidArgumentException('$predicate is not valid predicate.');
    }
}

function not($predicate)
{
    return predicate($predicate)->not();
}

function always()
{
    return ConstantPredicate::always();
}

function never()
{
    return ConstantPredicate::never();
}

function unpack($arguments)
{
    return count($arguments) === 1 && is_array(reset($arguments))
        ? reset($arguments)
        : $arguments;
}

function all(...$predicates)
{
    $predicates = unpack($predicates);
    return new AllOfPredicate(...array_map(predicate::class, $predicates));
}

function any(...$predicates)
{
    $predicates = unpack($predicates);
    return new AnyOfPredicate(...array_map(predicate::class, $predicates));
}
